function fser = fourier_ser(y, t, To, K);
% FOURIER_SER evaluates the Fourier series coefficients.
%
%   x(t) ~ A0 + sum_{k=0}^{K} Ak * cos((2*pi/To)*k) + Bk * sin((2*pi/To)*k)  
%
%   USAGE: [A, B, X, yr] = fourier_ser(y, t, To, K);
%
%   Signal has to be uniformly sampled, where time difference coresponst with 1/fs.
%
%   IN:
%   y - input signal vector evaluated at time points "t"
%   t - time points vector
%   To - signal period
%   K  - number of harmonics
%
%   OUT:
%   fser - structure of elements:
%   A - vector of Aks [A0 A1 A2 .... Ak] (A0 - DC value)
%   B - vector of Bks [0  B1 B2 .... Bk] (B0 = 0)
%   X - Fourier series spectral(DC value + harmonics) [A0, sqrt(An.^2 + Bn.^2)];
%   yr - Fourere series approximation of the signal
%   

if (nargin < 4), Ki = 10;
else, Ki = K;
endif

% Series
N = numel(y);
dt = t(2) - t(1);
to_bin = find(t >= To, 1, "first");
fo = 1/To;

% DC value 
A0 = (1/To)*sum(y(1:to_bin)*dt);

% Harmonics
An = zeros(1, Ki);
Bn = zeros(1, Ki);

for k=1:Ki
  An(k) = (2/To)*sum(y(1:to_bin).*cos(2*pi*k*fo*t(1:to_bin))*dt);
  Bn(k) = (2/To)*sum(y(1:to_bin).*sin(2*pi*k*fo*t(1:to_bin))*dt);  
endfor

%yser = zeros(1,numel(t));
%for n=1:numel(t)
%  yy = 0;
%  for k=1:Ki
%    yy = yy + An(k)*cos(2*pi*fo*k*t(n)) + Bn(k)*sin(2*pi*fo*k*t(n));
%  endfor
%  yser(n) = yy;
%endfor
%yser = A0 + yser;


if (Ki == 1)
  yser = A0 + (An.*cos(2*pi*fo*t) + Bn*sin(2*pi*fo*t));
else
  yser = A0 + sum(An'.*cos((1:numel(An))'*2*pi*fo.*t) + Bn'.*sin((1:numel(Bn))'*2*pi*fo.*t));
endif

fser.A = [A0, An];
fser.B = [0,  Bn];
fser.X = [A0, sqrt(An.^2 + Bn.^2)];
fser.yr = yser;
  

endfunction